# Code used in simulations for Northrup et al. 

# 2 conditional simulation with matched and mismatched availability samples

# Load needed libraries

library(arm)
library(raster)
library(spatstat)
require(grDevices)
library(survival)
library(geoR)

# Simualte landscape as a Gaussian Random field

phi=c(.001,100,1000)

x.001<-grf(10000, grid="reg", nx=100, ny=100, xlims=c(0,1200), ylims=c(0,1200), cov.pars=c(1,phi[1]))
# x10<-grf(10000, grid="reg", nx=100, ny=100, xlims=c(0,1200), ylims=c(0,1200), cov.pars=c(1,phi[2]))
# x100<-grf(10000, grid="reg", nx=100, ny=100, xlims=c(0,1200), ylims=c(0,1200), cov.pars=c(1,phi[3]))

# Convert covariate to raster

x.ras<-raster(x.001)
# x.ras<-raster(x10)
# x.ras<-raster(x100)


# Create a polygon and draw random locations within it

c1<-c(300,300)
c2<-c(300,900)
c3<-c(900,900)
c4<-c(900,300)
c5<-c(300,300)

coords<-rbind(c1,c2,c3,c4,c5)

poly<-Polygon(coords)

rand.num=500

rand<-spsample(poly, rand.num, "random")

# Add buffers of radius 100m and 200m around points 
coordinates<-rand@coords

polys<-list()

for(i in 1:rand.num){
	buff<-disc(radius=100, centre=c(coordinates[i,1], coordinates[i,2]))
	buffpoly<-Polygon(rbind(cbind(buff$bdry[[1]]$x, y=buff$bdry[[1]]$y), c(buff$bdry[[1]]$x[1],y=buff$bdry[[1]]$y[1])))
	polys<-c(polys, buffpoly)
}

polys.2<-list()

for(i in 1:rand.num){
	buff<-disc(radius=200, centre=c(coordinates[i,1], coordinates[i,2]))
	buffpoly<-Polygon(rbind(cbind(buff$bdry[[1]]$x, y=buff$bdry[[1]]$y), c(buff$bdry[[1]]$x[1],y=buff$bdry[[1]]$y[1])))
	polys.2<-c(polys.2, buffpoly)
}

# Within each buffer simulate as in simple simulation

lam.max=4

a=31415.93

N<-rpois(1, lam.max*a)

# Generate N points per buffer

pois.points<-list()

for(i in 1:rand.num){
	pois.points[[i]]<-spsample(polys[[i]], N, "random")
}
	
# Extract x values for all points
x.extract<-list()
x.coords<-list()


for(i in 1:rand.num){
	x.extract[[i]]<-extract(x.ras, pois.points[[i]])
	x.coords[[i]]<-pois.points[[i]]@coords
}

# Calculate mean and standard deviation for scaling

x.bar=cellStats(x.ras, 'mean')
x.std=cellStats(x.ras, 'sd')

x.scale<-list()
x.mat<-list()

for(i in 1:rand.num){
	x.scale[[i]]<-(x.extract[[i]]-x.bar)/x.std
	x.mat[[i]]<-cbind(rep(1,length(x.scale[[i]])), x.scale[[i]])
}

x.mat<-list()

for(i in 1:rand.num){
	x.mat[[i]]<-cbind(rep(1,length(x.extract[[i]])), x.extract[[i]])
}


# Vector of betas- Note first value is intercept, and can be used to alter the number of locations

beta<-c(-3,1)

# Calculate lambda at each point 

lam<-list()

for(i in 1:rand.num){
	lam[[i]]<-exp(x.mat[[i]]%*%beta)
}

# Calcualte probability and keep certain locations

p<-list()
dec<-list()
	
	for(i in 1:rand.num){
	p[[i]]<-lam[[i]]/lam.max
	dec[[i]]<-rbinom(N, 1, p[[i]])
}


# Select 1 used point from each buffer

x.u<-NULL
strat.u<-NULL


for(i in 1:rand.num){
	used.samp<-x.mat[[i]][dec[[i]]==1,]
	sample.num<-sample(length(used.samp[,1]),1)
	x.u[i]<-used.samp[sample.num,2]
	strat.u[i]<-i
}

# Draw 1,000 random locations within each buffer
# Sample from poly.2 to examine mismatched availability

pois.points.a<-list()

for(i in 1:rand.num){
	pois.points.a[[i]]<-spsample(polys[[i]], 1000, "random")
}
	
# Extract x values for all points
x.extract.a<-list()

for(i in 1:rand.num){
	x.extract.a[[i]]<-extract(x.ras, pois.points.a[[i]])
}

# Scale

x.scale.a<-list()
x.mat.a<-list()

for(i in 1:rand.num){
	x.scale.a[[i]]<-(x.extract.a[[i]]-x.bar)/x.std
	x.mat.a[[i]]<-cbind(rep(1,length(x.scale.a[[i]])), x.scale.a[[i]])
}

# Fit RSFs- to reproduce curves in manuscript, rerun simulation for multiple values of pts.a

iter=500	#controls number of RSFs to fit

pts.a=10	#controls number of available locations to draw within each buffer for each iteration 

# Create vector to store results

beta.1<-rep(0,iter)

# Run simulation

for(j in 1:iter){
	x.u<-NULL
	strat.u<-NULL

	for(i in 1:rand.num){
		used.samp<-x.mat[[i]][dec[[i]]==1,]
		sample.num<-sample(length(used.samp[,1]),1)
		x.u[i]<-used.samp[sample.num,2]
		strat.u[i]<-i
	}

	# Sample certain number of locations within each buffer
	avail.samp<-pts.a

	rand.points<-list()

	for(i in 1:rand.num){
		rand.points[[i]]<-spsample(polys.2[[i]], avail.samp, "random")
	}
	
	x.a<-NULL
	strat.a<-NULL


	for(i in 1:rand.num){
		x.a<-append(x.a, extract(x.ras, rand.points[[i]]))
		strat.a<-append(strat.a, rep(i, avail.samp))
	}

	x<-append(x.u, x.a)
	y<-append(rep(1,length(x.u)), rep(0, length(x.a)))
	strat<-append(strat.u, strat.a)

	model<-clogit(y~x+strata(strat))

	beta.1[j]<-model$coefficients
}

# Calculate expectation and simulation envelope

expec<-mean(beta.1)
sim.env.low<-quantile(beta.1, 0.025)
sim.env.up<-quantile(beta.1, 0.975)